/****  xfilter.c  ****/

/**********************************************************************
*  Copyright (c) 1991, 1992 Iris Computing Laboratories.
*
*  This software is provided for demonstration purposes only.  As
*  freely-distributed, modifiable source code, this software carries
*  absolutely no warranty.  Iris Computing Laboratories disclaims
*  all warranties for this software, including any implied warranties
*  of merchantability and fitness, and shall not be liable for
*  damages of any type resulting from its use.
*  Permission to use, copy, modify, and distribute this source code
*  for any purpose and without fee is hereby granted, provided that
*  the above copyright and this permission notice appear in all copies
*  and supporting documentation, and provided that Iris Computing
*  Laboratories not be named in advertising or publicity pertaining
*  to the redistribution of this software without specific, written
*  prior permission.
**********************************************************************/


/********************************************************************
`xfilter' implements a simple UNIX filter application where input
and output are X-related data repositories.  It is composed of the
following modules:
	`xfilter.c'		-- widget creation for the top-level interface,
					   plus callbacks and support functions.
	`xfilter.sel.c' -- the ICCCM-based selection and filter routines.
	`xfilter.hlp.t'	-- a module for providing help system text.
	`help.c'		-- a generalized help system.
	`motif.c'		-- OSF/Motif miscellaneous functions.

See these modules for more details.

`xfilter' allows you to specify an X-related source and target
for traditional UNIX filter operations such as `grep', `sed',
`spell', `tr', etc.  Rather than choose the filter from a menu,
`xfilter' asks you to type the filter's name into the text-edit
window labeled "Filter:".  This approach is more general in both
the range of filters that you can use and the potential for
specifying options.  If the filter area is null/empty, a simple
data exchange is performed from the input area to the output area.

Although `xfilter' includes support for files, it makes no effort
to optimize file-related special conditions.  All file-related
operations take place via temporary storage in dynamic memory.
`xfilter' is NOT a GUI alternative to cp(1) for copying files.
********************************************************************/

/*
NOTE:  The XmNmargin{Width,Height} resources that are hard-coded
below could be moved to a resource file, or included in the
fallback resources, but it's unlikely that anyone would ever
want to change them.
*/


#include "xfilter.h"
#include "xfilter.sel.h"
#include "help.h"
#include "motif.h"

#include "xfilter.hlp.t"		/* the help text */


/*
Private functions:
*/

static void print_help_info();
static void initialize_shell_icons();
static void handle_close_buttons();
static void create_main_window();
static Widget create_menu_bar();
static void create_dialogs();
static void dispatch_filter_operation();


/*
Private callbacks:
*/

static void Dismiss(), FileDismiss(), Filter(), SetFile();
static void SetInputOption(), SetOutputOption();
static void OverwriteOK(), OverwriteCancel();
static void Close(), UnmanageWindow(), UnmapWindow();


/*
Private globals:
*/

static Pixmap icon_pixmap;

static int input_type = xfilter_PRIMARY;
static int output_type = xfilter_TEXTWIN;

static Widget topLevel, fileShell, messageDialog, overwriteDialog;
static Widget filterText, textWindowText, fileInputText, fileOutputText;

static String fallback_resources[] = {
/*	"*optionBox.marginWidth: 5",
	"*filterBox.marginWidth: 5",
	"*filterBox.marginHeight: 5",
	"*inputOption.marginWidth: 0",
	"*fileDismissButton.marginWidth: 0",*******/
	"*textWindowText.rows: 8",
	"*textWindowText.columns: 60",
	NULL
};


/*
Public globals (shared with `xfilter.sel.c'):
*/

XmStringCharSet char_set = (XmStringCharSet) XmSTRING_DEFAULT_CHARSET;



/*
main() creates a main top-level window, plus secondary windows for
specifying filenames, pop-up dialogs, etc., by calling several
secondary functions.
*/

void main(argc, argv)
int argc;
char *argv[];
{
	XtAppContext app;

	if (argc == 2 && strcmp(argv[1], "-help") == 0) {
		print_help_info();
		exit(0);
	}
	topLevel = XtAppInitialize(&app, xfilter_APP_CLASS,
		(XrmOptionDescList) NULL, 0,
		&argc, argv, fallback_resources, (ArgList) NULL, 0);
	create_main_window();
	create_dialogs();
	help_create_dialog(topLevel, xfilter_APP_CLASS,
		xfilter_help_rows, xfilter_help_columns, char_set);
	initialize_shell_icons();
	XtRealizeWidget(topLevel);
	XtRealizeWidget(fileShell);
	help_realize();
	initialize_selection_structures(topLevel);
	handle_close_buttons();
#ifdef SCREEN_SHOT
	while (True) {
		XEvent event;

		XtAppNextEvent(app, &event);
		if (event.type == ButtonPress && event.xbutton.button == 2) {
			XUngrabPointer(XtDisplay(topLevel), CurrentTime);
			XUngrabKeyboard(XtDisplay(topLevel), CurrentTime);
		}
		XtDispatchEvent(&event);
	}
#else
	XtAppMainLoop(app);
#endif
}	/* main */


/*
Private callback functions:
*/

/*
Dismiss() terminates the application after freeing dynamic resources.
*/
/*ARGSUSED*/
static void Dismiss(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	free_selection_structures();
	XFreePixmap(XtDisplay(w), icon_pixmap);
	exit(0);
}	/* Dismiss */


/*
Close(), at present, is similar to Dismiss().
*/
/*ARGSUSED*/
static void Close(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	free_selection_structures();
	XFreePixmap(XtDisplay(topLevel), icon_pixmap);
	exit(0);
}	/* Close */


/*
FileDismiss() simply removes the dialog box for filename entry.
*/
/*ARGSUSED*/
static void FileDismiss(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	XtPopdown(fileShell);
/*	XtUnmapWidget(fileShell); ** for XtAppCreateShell() **/
}	/* FileDismiss */


/*
Filter() simply calls the top-level dispatcher.
*/
/*ARGSUSED*/
static void Filter(w, client_data, call_data)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *call_data;
{
	if (output_type == xfilter_FILE) {
		char *file_spec = XmTextGetString(fileOutputText);

		if (!*file_spec)
			user_message("No output file specified.");
		else if (file_size(file_spec) != -1)
			XtManageChild(overwriteDialog);
		else
			dispatch_filter_operation(w);
		XtFree(file_spec);
	}
	else
		dispatch_filter_operation(w);
}	/* Filter */


/*
SetFile() simply maps the dialog box for filename entry.
*/
/*ARGSUSED*/
static void SetFile(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	XtPopup(fileShell, XtGrabNone);
/*	XtMapWidget(fileShell); ** for XtAppCreateShell() **/
}	/* SetFile */


/*
SetInputOption() records user selections for input.
*/
/*ARGSUSED*/
static void SetInputOption(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	input_type = (int) client_data;
}	/* SetInputOption */


/*
SetOutputOption() records user selections for input.
*/
/*ARGSUSED*/
static void SetOutputOption(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	output_type = (int) client_data;
}	/* SetOutputOption */


/*
OverwriteOK() handles the appropriate action if a user
chooses the "OK" button during a "Filter" operation
that leads to the file-overwrite warning.
*/
/*ARGSUSED*/
static void OverwriteOK(w, client_data, call_data)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *call_data;
{
	dispatch_filter_operation(w);
}	/* OverwriteOK */


/*
OverwriteCancel() handles the appropriate action if a user
chooses the "Cancel" button during a "Filter" operation
that leads to the file-overwrite warning.
*/
/*ARGSUSED*/
static void OverwriteCancel(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	/* do nothing */
}	/* OverwriteCancel */


/*
UnmapWindow() provides "Close" button functionality for shells.
*/
/*ARGSUSED*/
static void UnmapWindow(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	XtUnmapWidget((Widget) client_data);
}	/* UnmapWindow */


/*
UnmanageWindow() provides "Close" button functionality for dialog shells.
*/
/*ARGSUSED*/
static void UnmanageWindow(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
	XtUnmanageChild((Widget) client_data);
}	/* UnmanageWindow */


/*
Public functions:
*/

/*
user_message() sets the message dialog box to have the
specified message and then maps the dialog box.
*/

void user_message(msg)
char *msg;
{
	Arg args[1];
	XmString string;

	string = XmStringCreateLtoR(msg, char_set);
	XtSetArg(args[0], XmNmessageString, string);
	XtSetValues(messageDialog, args, 1);
	XmStringFree(string);
	XtManageChild(messageDialog);
}	/* user_message */


/*
Without further commenting, the following functions
return the state of selected variables:
*/

int get_input_type()
{
	return input_type;
}	/* get_input_type */

int get_output_type()
{
	return output_type;
}	/* get_output_type */

Widget get_input_text_widget()
{
	return fileInputText;
}	/* get_input_text_widget */

Widget get_output_text_widget()
{
	return fileOutputText;
}	/* get_output_text_widget */

Widget get_filter_text_widget()
{
	return filterText;
}	/* get_filter_text_widget */

Widget get_text_win_widget()
{
	return textWindowText;
}	/* get_text_win_widget */


/*
Support functions:
*/

/*
create_main_window() delegates menu bar creation to subordinate
routines; otherwise, most widgets are created directly.
*/

static void create_main_window()
{
	static menu_entry input_menu[] = {
		{menu_ENTRY, "Primary selection", "inputPrimary", SetInputOption,
			(XtPointer) xfilter_PRIMARY, NULL, NULL},
		{menu_ENTRY, "Cut-buffer 0", "inputCutBuffer", SetInputOption,
			(XtPointer) xfilter_CUT_BUFFER, NULL, NULL},
		{menu_ENTRY, "X clipboard", "inputXClipboard", SetInputOption,
			(XtPointer) xfilter_XCLIPBOARD, NULL, NULL},
		{menu_ENTRY, "From file", "inputFile", SetInputOption,
			(XtPointer) xfilter_FILE, NULL, NULL},
		{menu_END, NULL, NULL, NULL, NULL, NULL, NULL},
	};
	static menu_entry output_menu[] = {
		{menu_ENTRY, "Cut-buffer 0", "outputCutBuffer", SetOutputOption,
			(XtPointer) xfilter_CUT_BUFFER, NULL, NULL},
		{menu_ENTRY, "X clipboard", "outputXClipboard", SetOutputOption,
			(XtPointer) xfilter_XCLIPBOARD, NULL, NULL},
		{menu_ENTRY, "Text window", "outputTextWindow", SetOutputOption,
			(XtPointer) xfilter_TEXTWIN, NULL, NULL},
		{menu_ENTRY, "To file", "outputFile", SetOutputOption,
			(XtPointer) xfilter_FILE, NULL, NULL},
		{menu_END, NULL, NULL, NULL, NULL, NULL, NULL},
	};
	Arg args[10];
	int i;
	Widget mainWindow, menuBar, optionFrame, optionBox,
		textFrame, textWindow, inputMenu, outputMenu,
		filterButton, inputOption, outputOption,
		filterFrame, filterBox, filterLabel;

	i = 0;
	XtSetArg(args[i], XmNhorizontalSpacing, (XtArgVal) 5); i++;
	XtSetArg(args[i], XmNverticalSpacing, (XtArgVal) 5); i++;
	mainWindow = XtCreateManagedWidget("mainWindow",
		xmFormWidgetClass, topLevel, args, i);

	menuBar = create_menu_bar(mainWindow);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopWidget, (XtArgVal) menuBar); i++;
	optionFrame = XtCreateManagedWidget("optionFrame",
		xmFrameWidgetClass, mainWindow, args, i);

	i = 0;
	XtSetArg(args[i], XmNmarginWidth, (XtArgVal) 5); i++;
	XtSetArg(args[i], XmNorientation, (XtArgVal) XmHORIZONTAL); i++;
	optionBox = XtCreateManagedWidget("optionBox",
		xmRowColumnWidgetClass, optionFrame, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopWidget, (XtArgVal) optionFrame); i++;
	filterFrame = XtCreateManagedWidget("filterFrame",
		xmFrameWidgetClass, mainWindow, args, i);

	i = 0;
	XtSetArg(args[i], XmNmarginWidth, (XtArgVal) 5); i++;
	XtSetArg(args[i], XmNmarginHeight, (XtArgVal) 5); i++;
	filterBox = XtCreateManagedWidget("filterBox",
		xmFormWidgetClass, filterFrame, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Filter", char_set)); i++;
	filterButton = XtCreateManagedWidget("filterButton",
		xmPushButtonWidgetClass, filterBox, args, i);
	XtAddCallback(filterButton, XmNactivateCallback, Filter, NULL);
	
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNleftWidget, (XtArgVal) filterButton); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR(" Filter: ", char_set)); i++;
	filterLabel = XtCreateManagedWidget("filterLabel",
		xmLabelWidgetClass, filterBox, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNleftWidget, (XtArgVal) filterLabel); i++;
	filterText = XtCreateManagedWidget("filterText",
		xmTextWidgetClass, filterBox, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopWidget, (XtArgVal) filterFrame); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	textFrame = XtCreateManagedWidget("textFrame",
		xmFrameWidgetClass, mainWindow, args, i);

	i = 0;
	textWindow = XtCreateManagedWidget("textWindow",
		xmScrolledWindowWidgetClass, textFrame, args, i);
	i = 0;
	XtSetArg(args[i], XmNeditMode, (XtArgVal) XmMULTI_LINE_EDIT); i++;
	XtSetArg(args[i], XmNautoShowCursorPosition, (XtArgVal) True); i++;
	XtSetArg(args[i], XmNscrollHorizontal, (XtArgVal) True); i++;
	XtSetArg(args[i], XmNscrollVertical, (XtArgVal) True); i++;
	textWindowText = XtCreateManagedWidget("textWindowText",
		xmTextWidgetClass, textWindow, args, i);

	inputMenu = XmCreatePulldownMenu(optionBox, "inputMenu", NULL, 0);
	create_menus(inputMenu, input_menu, char_set);
	i = 0;
	XtSetArg(args[i], XmNmarginWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Input:", char_set)); i++;
	XtSetArg(args[i], XmNsubMenuId, inputMenu); i++;
	XtSetArg(args[i], XmNmenuHistory, input_menu[0].id); i++;
	inputOption = XmCreateOptionMenu(optionBox, "inputOption", args, i);
	XtManageChild(inputOption);

	outputMenu = XmCreatePulldownMenu(optionBox, "outputMenu", NULL, 0);
	create_menus(outputMenu, output_menu, char_set);
	i = 0;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Output:", char_set)); i++;
	XtSetArg(args[i], XmNsubMenuId, outputMenu); i++;
	XtSetArg(args[i], XmNmenuHistory, output_menu[2].id); i++;
	outputOption = XmCreateOptionMenu(optionBox, "outputOption", args, i);
	XtManageChild(outputOption);
}	/* create_main_window */


/*
create_menu_bar() creates the application-level menu bar,
including pull-down menus and menu entries.
*/

static Widget create_menu_bar(parent)
Widget parent;
{
	static menu_entry file_menu[] = {
		{menu_ENTRY, "Set File(s)...", "menuSetFileButton", SetFile,
			NULL, NULL, NULL},
		{menu_ENTRY, "Dismiss", "menuExitButton", Dismiss,
			NULL, NULL, NULL},
		{menu_END, NULL, NULL, NULL, NULL, NULL, NULL},
	};
	static menu_entry menus[] = {
		{menu_SUBMENU, "File", "menuFileButton", NULL, NULL, file_menu, NULL},
		{menu_END, NULL, NULL, NULL, NULL, NULL, NULL},
	};
	static menu_entry help_menu[] = {
		{menu_ENTRY, "General...", "menuGenHelpButton",
			Help, (XtPointer) 0, NULL, NULL},
		{menu_ENTRY, "Files...", "menuFileHelpButton",
			Help, (XtPointer) 1, NULL, NULL},
		{menu_ENTRY, "Info...", "menuInfoHelpButton",
			Help, (XtPointer) 2, NULL, NULL},
		{menu_END, NULL, NULL, NULL, NULL, NULL, NULL},
	};
	Arg args[10];
	int i;
	Widget menuBar;

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	menuBar = XmCreateMenuBar(parent, "menuBar", args, i);
	XtManageChild(menuBar);
	create_menus(menuBar, menus, char_set);
	help_create_pulldown_menu(menuBar, help_menu, xfilter_help_data, char_set);
	return menuBar;
}	/* create_menu_bar */


/*
create_dialogs() creates the dialog box used for setting
filenames for the input and output file options, and the
user-message dialog box.
*/

static void create_dialogs()
{
	Arg args[10];
	int i;
	Dimension height, margin_height;
	Widget filePane, fileBox, fileLabel, fileInputLabel, fileOutputLabel,
		fileInputBox, fileOutputBox, fileCommandBox, fileDismissButton;

	i = 0;
	XtSetArg(args[i], XmNallowShellResize, (XtArgVal) True); i++;
	XtSetArg(args[i], XmNmappedWhenManaged, (XtArgVal) False); i++;
	fileShell = XtCreatePopupShell("fileShell", topLevelShellWidgetClass,
		topLevel, args, i);
/*	fileShell = XtAppCreateShell(NULL, xfilter_APP_CLASS,
		topLevelShellWidgetClass, XtDisplay(topLevel), args, i);*/
	i = 0;
	XtSetArg(args[i], XmNsashWidth, (XtArgVal) 1); i++;
	XtSetArg(args[i], XmNsashHeight, (XtArgVal) 1); i++;
	filePane = XtCreateManagedWidget("filePane",
		xmPanedWindowWidgetClass, fileShell, args, i);
	i = 0;
	XtSetArg(args[i], XmNhorizontalSpacing, (XtArgVal) 5); i++;
	XtSetArg(args[i], XmNverticalSpacing, (XtArgVal) 5); i++;
	fileBox = XtCreateManagedWidget("fileBox",
		xmFormWidgetClass, filePane, args, i);
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Set filenames for file input/output options:",
			char_set)); i++;
	XtSetArg(args[i], XmNalignment, (XtArgVal) XmALIGNMENT_CENTER); i++;
	fileLabel = XtCreateManagedWidget("fileLabel",
		xmLabelWidgetClass, fileBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopWidget, (XtArgVal) fileLabel); i++;
	fileInputBox = XtCreateManagedWidget("fileInputBox",
		xmFormWidgetClass, fileBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNtopWidget, (XtArgVal) fileInputBox); i++;
	fileOutputBox = XtCreateManagedWidget("fileOutputBox",
		xmFormWidgetClass, fileBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNborderWidth, (XtArgVal) 0); i++;
	XtSetArg(args[i], XmNorientation, (XtArgVal) XmHORIZONTAL); i++;
	fileCommandBox = XtCreateManagedWidget("fileCommandBox",
		xmRowColumnWidgetClass, filePane, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Input:  ", char_set)); i++;
	fileInputLabel = XtCreateManagedWidget("fileInputLabel",
		xmLabelWidgetClass, fileInputBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNleftWidget, (XtArgVal) fileInputLabel); i++;
	fileInputText = XtCreateManagedWidget("fileInputText",
		xmTextWidgetClass, fileInputBox, args, i);

	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Output: ", char_set)); i++;
	fileOutputLabel = XtCreateManagedWidget("fileOutputLabel",
		xmLabelWidgetClass, fileOutputBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNleftAttachment, (XtArgVal) XmATTACH_WIDGET); i++;
	XtSetArg(args[i], XmNrightAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNtopAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNbottomAttachment, (XtArgVal) XmATTACH_FORM); i++;
	XtSetArg(args[i], XmNleftWidget, (XtArgVal) fileOutputLabel); i++;
	fileOutputText = XtCreateManagedWidget("fileOutputText",
		xmTextWidgetClass, fileOutputBox, args, i);

	i = 0;
	XtSetArg(args[i], XmNlabelString,
		XmStringCreateLtoR("Dismiss", char_set)); i++;
	XtSetArg(args[i], XmNmarginWidth, (XtArgVal) 0); i++;
	fileDismissButton = XtCreateManagedWidget("fileDismissButton",
		xmPushButtonWidgetClass, fileCommandBox, args, i);
	XtAddCallback(fileDismissButton,
		XmNactivateCallback, FileDismiss, NULL);

	i = 0;
	XtSetArg(args[i], XmNmarginHeight, &margin_height); i++;
	XtGetValues(fileCommandBox, args, i);
	i = 0;
	XtSetArg(args[i], XmNheight, &height); i++;
	XtGetValues(fileDismissButton, args, i);
	i = 0;
	XtSetArg(args[i], XmNpaneMinimum,
		(XtArgVal) (height + (margin_height * 2))); i++;
	XtSetArg(args[i], XmNpaneMaximum,
		(XtArgVal) (height + (margin_height * 2))); i++;
	XtSetValues(fileCommandBox, args, i);
	remove_sash_traversal(filePane);

	i = 0;
	XtSetArg(args[i], XmNdialogTitle,
		XmStringCreateLtoR("xfilter", char_set)); i++;
	XtSetArg(args[i], XmNokLabelString,
		XmStringCreateLtoR("  OK  ", char_set)); i++;
	messageDialog = (Widget) XmCreateInformationDialog(topLevel,
		"messageDialog", args, i);
	XtUnmanageChild((Widget) XmMessageBoxGetChild(messageDialog,
		XmDIALOG_CANCEL_BUTTON));
	XtUnmanageChild((Widget) XmMessageBoxGetChild(messageDialog,
		XmDIALOG_HELP_BUTTON));

	i = 0;
	XtSetArg(args[i], XmNdialogTitle,
		XmStringCreateLtoR("xfilter", char_set)); i++;
	XtSetArg(args[i], XmNmessageString,
		XmStringCreateLtoR("File already exists!", char_set)); i++;
	XtSetArg(args[i], XmNokLabelString,
		XmStringCreateLtoR("Overwrite File", char_set)); i++;
	overwriteDialog = (Widget) XmCreateQuestionDialog(topLevel,
		"overwriteDialog", args, i);
	XtUnmanageChild((Widget) XmMessageBoxGetChild(overwriteDialog,
		XmDIALOG_HELP_BUTTON));
	XtAddCallback(overwriteDialog, XmNokCallback,
		OverwriteOK, NULL);
	XtAddCallback(overwriteDialog, XmNcancelCallback,
		OverwriteCancel, NULL);
}	/* create_dialogs */


/*
print_help_info() prints a short message about how `xfilter' works.
*/

static void print_help_info()
{
	printf(
"\nusage:  xfilter\n\
\n\
`xfilter' applies a UNIX filter, as specified in the text\n\
entry area labeled \"Filter:\", (with or without options)\n\
to common X data storage locations.  It uses the specified\n\
input and output devices/locations as analogs to standard\n\
input and standard output files in a command line execution\n\
of the filter.  If no filter is specified, pressing the\n\
\"Filter\" button simply copies the input to the output.\n\
For example, setting \"Input:\" to \"Primary selection\",\n\
\"Output:\" to \"Text window\", and \"Filter:\" to \"spell\", will\n\
apply spell(1) to the selected text, reporting questionable\n\
word spellings in `xfilter's scrollable text window.\n\
");
}	/* print_help_info */


/*
initialize_shell_icons() sets the icon(s).
*/

static void initialize_shell_icons()
{

#include "xfilter.icon"

	Arg args[1];

	icon_pixmap = XCreateBitmapFromData(XtDisplay(topLevel),
		RootWindowOfScreen(XtScreen(topLevel)), xfilter_bits,
		xfilter_width, xfilter_height);
	XtSetArg(args[0], XtNiconPixmap, (XtArgVal) icon_pixmap);
	XtSetValues(topLevel, args, 1);
	XtSetValues(help_shell(), args, 1);
	XtSetValues(fileShell, args, 1);
}	/* initialize_shell_icons */


/*
dispatch_filter_operation() is a top-level dispatcher that
makes the appropriate filter request.
*/

static void dispatch_filter_operation(w)
Widget w;
{
	char *file_spec;

	switch (input_type) {
		case xfilter_PRIMARY:
			filter_primary_selection(w);
			break;
		case xfilter_CUT_BUFFER:
			filter_cut_buffer(w);
			break;
		case xfilter_XCLIPBOARD:
			filter_clipboard_selection(w);
			break;
		case xfilter_FILE:
			file_spec = XmTextGetString(fileInputText);
			if (!*file_spec) {
				user_message("No input file specified.");
				XtFree(file_spec);
				break;
			}
			filter_file(w, file_spec);
			XtFree(file_spec);
			break;
		default:
			break;
	}
}	/* dispatch_filter_operation */


/*
handle_close_buttons() accommodates the "Close" buttons.  The actions
should be hard-coded -- the user should not be allowed to override the
XmUNMAP default for pop-ups.
*/

static void handle_close_buttons()
{
	Atom xfilter_DELETE_WINDOW;
	Arg arg;

	xfilter_DELETE_WINDOW = XmInternAtom(XtDisplay(topLevel),
		"WM_DELETE_WINDOW", False);
	XtSetArg(arg, XmNdeleteResponse, (XtArgVal) XmDO_NOTHING);
	XtSetValues(topLevel, &arg, 1);
	XtSetValues(help_shell(), &arg, 1);
	XtSetValues(fileShell, &arg, 1);
	XtSetValues(XtParent(messageDialog), &arg, 1);
	XtSetValues(XtParent(overwriteDialog), &arg, 1);
	XmAddWMProtocolCallback(topLevel, xfilter_DELETE_WINDOW,
		Close, NULL);
	XmAddWMProtocolCallback(help_shell(), xfilter_DELETE_WINDOW,
		UnmapWindow, (caddr_t) help_shell());
	XmAddWMProtocolCallback(fileShell, xfilter_DELETE_WINDOW,
		UnmapWindow, (caddr_t) fileShell);
	XmAddWMProtocolCallback(XtParent(messageDialog), xfilter_DELETE_WINDOW,
		UnmanageWindow, (caddr_t) messageDialog);
	XmAddWMProtocolCallback(XtParent(overwriteDialog), xfilter_DELETE_WINDOW,
		UnmanageWindow, (caddr_t) overwriteDialog);
}	/* handle_close_buttons */

